@@ -10,7 +10,7 @@ from ..config import Config
 from ..data_model import (TestCase, SedTaskRequirements, ExpectedSedReport, ExpectedSedDataSet, ExpectedSedPlot,
                           AlertType, OutputMedium)
 from ..exceptions import InvalidOutputsException, SkippedTestCaseException, TimeoutException, TestCaseException
-from ..utils import get_singularity_image_filename
+from ..utils import get_singularity_image_filename, simulation_results_isnan
 from ..warnings import IgnoredTestCaseWarning, SimulatorRuntimeErrorWarning, InvalidOutputsWarning
 from .utils import are_array_shapes_equivalent
 from biosimulators_utils.combine.data_model import CombineArchive, CombineArchiveContentFormatPattern  # noqa: F401
@@ -39,7 +39,6 @@ import os
 import re
 import shutil
 import subprocess
-import tempfile
 import types  # noqa: F401
 import warnings
 
@@ -289,11 +288,12 @@ class SimulatorCanExecutePublishedProject(TestCase):
 
         return True
 
-    def eval(self, specifications, synthetic_archives_dir=None, dry_run=False, cli=None):
+    def eval(self, specifications, working_dirname, synthetic_archives_dir=None, dry_run=False, cli=None):
         """ Evaluate a simulator's performance on a test case
 
         Args:
             specifications (:obj:`dict`): specifications of the simulator to validate
+            working_dirname (:obj:`str`): directory for temporary files for evaluating test case
             synthetic_archives_dir (:obj:`str`, optional): Directory to save the synthetic COMBINE/OMEX archives
                 generated by the test cases
             dry_run (:obj:`bool`): if :obj:`True`, do not use the simulator to execute COMBINE/OMEX archives.
@@ -316,18 +316,17 @@ class SimulatorCanExecutePublishedProject(TestCase):
         if dry_run:
             return
 
-        # create output directory for case
-        out_dir = tempfile.mkdtemp()
+        if not os.path.isdir(working_dirname):
+            os.makedirs(working_dirname)
 
         # pull image and execute COMBINE/OMEX archive for case
         try:
-            self.exec_sedml_docs_in_archive(specifications, out_dir, cli=cli)
+            self.exec_sedml_docs_in_archive(specifications, working_dirname, cli=cli)
 
         except Exception as exception:
-            if os.path.isdir(out_dir) and os.getenv('CI', 'false').lower() in ['1', 'true']:
-                subprocess.run(['sudo', 'chown', '{}:{}'.format(os.getuid(), os.getgid()), '-R', out_dir], check=True)
+            if os.path.isdir(working_dirname) and os.getenv('CI', 'false').lower() in ['1', 'true']:
+                subprocess.run(['sudo', 'chown', '{}:{}'.format(os.getuid(), os.getgid()), '-R', working_dirname], check=True)
 
-            shutil.rmtree(out_dir)
             if self.runtime_failure_alert_type == AlertType.exception:
                 raise
             else:
@@ -338,7 +337,7 @@ class SimulatorCanExecutePublishedProject(TestCase):
         errors = []
 
         # check expected outputs created: reports
-        if not os.path.isfile(os.path.join(out_dir, get_config().H5_REPORTS_PATH)):
+        if not os.path.isfile(os.path.join(working_dirname, get_config().H5_REPORTS_PATH)):
             errors.append('No reports were generated')
 
         else:
@@ -348,7 +347,7 @@ class SimulatorCanExecutePublishedProject(TestCase):
                 for data_set in expected_report.data_sets:
                     report.data_sets.append(DataSet(id=data_set.id, label=data_set.label))
                 try:
-                    report_results = report_reader.run(report, out_dir, expected_report.id, format=ReportFormat.h5)
+                    report_results = report_reader.run(report, working_dirname, expected_report.id, format=ReportFormat.h5)
                 except Exception:
                     errors.append('Report {} could not be read'.format(expected_report.id))
                     continue
@@ -397,7 +396,7 @@ class SimulatorCanExecutePublishedProject(TestCase):
                             errors.append('Data set {} of report {} does not have expected values'.format(
                                 data_set_id, expected_report.id))
 
-            report_ids = set(report_reader.get_ids(out_dir))
+            report_ids = set(report_reader.get_ids(working_dirname))
             expected_report_ids = set(report.id for report in self.expected_reports)
             extra_report_ids = report_ids.difference(expected_report_ids)
             if extra_report_ids:
@@ -409,8 +408,8 @@ class SimulatorCanExecutePublishedProject(TestCase):
                         '\n  '.join(sorted(extra_report_ids))), InvalidOutputsWarning)
 
         # check expected outputs created: plots
-        if os.path.isfile(os.path.join(out_dir, get_config().PLOTS_PATH)):
-            archive = biosimulators_utils.archive.io.ArchiveReader().run(os.path.join(out_dir, 'plots.zip'))
+        if os.path.isfile(os.path.join(working_dirname, get_config().PLOTS_PATH)):
+            archive = biosimulators_utils.archive.io.ArchiveReader().run(os.path.join(working_dirname, 'plots.zip'))
             plot_ids = set(os.path.splitext(file.archive_path)[0] for file in archive.files)
         else:
             plot_ids = set()
@@ -436,9 +435,6 @@ class SimulatorCanExecutePublishedProject(TestCase):
                 warnings.warn('Extra plots were produced:\n  {}'.format(
                     '\n  '.join(sorted(extra_plot_ids))), InvalidOutputsWarning)
 
-        # cleanup outputs
-        shutil.rmtree(out_dir)
-
         # raise errors
         if errors:
             raise InvalidOutputsException('\n\n'.join(errors))
@@ -501,11 +497,12 @@ class SyntheticCombineArchiveTestCase(TestCase):
         self.published_projects_test_cases = published_projects_test_cases or []
         self._published_projects_test_case = None
 
-    def eval(self, specifications, synthetic_archives_dir=None, dry_run=False, cli=None):
+    def eval(self, specifications, working_dirname, synthetic_archives_dir=None, dry_run=False, cli=None):
         """ Evaluate a simulator's performance on a test case
 
         Args:
             specifications (:obj:`dict`): specifications of the simulator to validate
+            working_dirname (:obj:`str`): directory for temporary files for evaluating test case
             synthetic_archives_dir (:obj:`str`, optional): Directory to save the synthetic COMBINE/OMEX archives
                 generated by the test cases
             dry_run (:obj:`bool`): if :obj:`True`, do not use the simulator to execute COMBINE/OMEX archives.
@@ -519,7 +516,8 @@ class SyntheticCombineArchiveTestCase(TestCase):
             :obj:`Exception`: if the simulator did not pass the test case
         """
         try:
-            return_value = self._eval(specifications, synthetic_archives_dir=synthetic_archives_dir, dry_run=dry_run, cli=cli)
+            return_value = self._eval(specifications, working_dirname,
+                                      synthetic_archives_dir=synthetic_archives_dir, dry_run=dry_run, cli=cli)
         except Exception as exception:
             if not isinstance(exception, TimeoutException) and self.REPORT_ERROR_AS_SKIP:
                 raise SkippedTestCaseException(str(exception))
@@ -528,11 +526,12 @@ class SyntheticCombineArchiveTestCase(TestCase):
 
         return return_value
 
-    def _eval(self, specifications, synthetic_archives_dir=None, dry_run=False, cli=None):
+    def _eval(self, specifications, working_dirname, synthetic_archives_dir=None, dry_run=False, cli=None):
         """ Evaluate a simulator's performance on a test case
 
         Args:
             specifications (:obj:`dict`): specifications of the simulator to validate
+            working_dirname (:obj:`str`): directory for temporary files for evaluating test case
             synthetic_archives_dir (:obj:`str`, optional): Directory to save the synthetic COMBINE/OMEX archives
                 generated by the test cases
             dry_run (:obj:`bool`): if :obj:`True`, do not use the simulator to execute COMBINE/OMEX archives.
@@ -545,7 +544,8 @@ class SyntheticCombineArchiveTestCase(TestCase):
         Raises:
             :obj:`Exception`: if the simulator did not pass the test case
         """
-        temp_dir = tempfile.mkdtemp()
+        if not os.path.isdir(working_dirname):
+            os.makedirs(working_dirname)
 
         # read curated archives and find one that is suitable for testing
         suitable_curated_archive = False
@@ -554,8 +554,9 @@ class SyntheticCombineArchiveTestCase(TestCase):
 
             # read archive
             curated_archive_filename = published_projects_test_case.filename
-            shared_archive_dir = os.path.join(temp_dir, 'archive')
-            os.mkdir(shared_archive_dir)
+            shared_archive_dir = os.path.join(working_dirname, 'archive')
+            if not os.path.isdir(shared_archive_dir):
+                os.makedirs(shared_archive_dir)
 
             curated_archive = CombineArchiveReader().run(curated_archive_filename, shared_archive_dir)
             curated_sed_docs = {}
@@ -582,32 +583,31 @@ class SyntheticCombineArchiveTestCase(TestCase):
             shutil.rmtree(shared_archive_dir)
 
         if not suitable_curated_archive:
-            shutil.rmtree(temp_dir)
             raise SkippedTestCaseException('No curated COMBINE/OMEX archives are available to generate archives for testing')
 
         expected_results_of_synthetic_archives = self.build_synthetic_archives(
             specifications, curated_archive, shared_archive_dir, curated_sed_docs)
         has_warnings = False
-        try:
-            for i_archive, expected_results_of_synthetic_archive in enumerate(expected_results_of_synthetic_archives):
-                if self._eval_synthetic_archive(specifications, expected_results_of_synthetic_archive, shared_archive_dir,
-                                                i_archive, synthetic_archives_dir=synthetic_archives_dir, dry_run=dry_run,
-                                                cli=cli):
-                    has_warnings = True
-        finally:
-            shutil.rmtree(temp_dir)
+        for i_archive, expected_results_of_synthetic_archive in enumerate(expected_results_of_synthetic_archives):
+            if self._eval_synthetic_archive(specifications, expected_results_of_synthetic_archive, shared_archive_dir,
+                                            i_archive, os.path.join(working_dirname, str(i_archive + 1)),
+                                            synthetic_archives_dir=synthetic_archives_dir, dry_run=dry_run,
+                                            cli=cli):
+                has_warnings = True
         return not has_warnings
 
     def _eval_synthetic_archive(self, specifications, expected_results_of_synthetic_archive, shared_archive_dir,
-                                i_synthetic_archive, synthetic_archives_dir=None, dry_run=False, cli=None):
+                                i_synthetic_archive, working_dirname, synthetic_archives_dir=None, dry_run=False, cli=None):
         synthetic_archive = expected_results_of_synthetic_archive.archive
         synthetic_sed_docs = expected_results_of_synthetic_archive.sed_documents
         is_success_expected = expected_results_of_synthetic_archive.is_success_expected
         environment = expected_results_of_synthetic_archive.environment
 
+        if not os.path.isdir(working_dirname):
+            os.makedirs(working_dirname)
+
         sedml_writer = SedmlSimulationWriter()
-        temp_dir = tempfile.mkdtemp()
-        synthetic_archive_filename = os.path.join(temp_dir, 'archive.omex')
+        synthetic_archive_filename = os.path.join(working_dirname, 'archive.omex')
         for location, sed_doc in synthetic_sed_docs.items():
             sedml_writer.run(sed_doc, os.path.join(shared_archive_dir, location))
         CombineArchiveWriter().run(synthetic_archive, shared_archive_dir, synthetic_archive_filename)
@@ -627,11 +627,10 @@ class SyntheticCombineArchiveTestCase(TestCase):
             shutil.copy(synthetic_archive_filename, export_synthetic_archive_filename)
 
         if dry_run:
-            shutil.rmtree(temp_dir)
             return False
 
         # use synthetic archive to test simulator
-        outputs_dir = os.path.join(temp_dir, 'outputs')
+        outputs_dir = os.path.join(working_dirname, 'outputs')
         config = Config()
         pull_docker_image = config.pull_docker_image
         user_to_exec_within_container = config.user_to_exec_in_simulator_containers
@@ -693,11 +692,9 @@ class SyntheticCombineArchiveTestCase(TestCase):
 
             succeeded = False
             if is_success_expected:
-                shutil.rmtree(temp_dir)
                 raise
 
         if succeeded and not is_success_expected:
-            shutil.rmtree(temp_dir)
             msg = 'The execution of the COMBINE/OMEX archive did not fail as expected'
             raise ValueError(msg)
 
@@ -1213,7 +1210,7 @@ class UniformTimeCourseTestCase(SingleMasterSedDocumentCombineArchiveTestCase):
         data = ReportReader().run(report, outputs_dir, os.path.join(doc_id, report.id))
 
         for data_set_data in data.values():
-            if numpy.any(numpy.isnan(data_set_data)):
+            if numpy.any(simulation_results_isnan(data_set_data)):
                 warnings.warn('The results produced by the simulator include `NaN`.', InvalidOutputsWarning)
                 has_warnings = True
                 break
